@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -81,12 +81,17 @@ class GenresDatabase:
         """
         orderby = App().settings.get_enum("orderby")
         order = " ORDER BY genres.name, "
-        if orderby == OrderBy.ARTIST:
+        if orderby == OrderBy.ARTIST_YEAR:
             order += " artists.sortname\
                      COLLATE NOCASE COLLATE LOCALIZED,\
                      albums.timestamp,\
                      albums.name\
                      COLLATE NOCASE COLLATE LOCALIZED"
+        elif orderby == OrderBy.ARTIST_TITLE:
+            order += " artists.sortname\
+                     COLLATE NOCASE COLLATE LOCALIZED,\
+                     albums.name\
+                     COLLATE NOCASE COLLATE LOCALIZED"
         elif orderby == OrderBy.NAME:
             order += " albums.name\
                      COLLATE NOCASE COLLATE LOCALIZED"
