@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -15,7 +15,7 @@ from gettext import gettext as _
 import itertools
 
 from lollypop.sqlcursor import SqlCursor
-from lollypop.define import App, StorageType
+from lollypop.define import App, StorageType, Type, LovedFlags
 from lollypop.utils import noaccents, make_subrequest
 
 
@@ -107,12 +107,14 @@ class TracksDatabase:
             @return track ids as [int]
         """
         with SqlCursor(self.__db) as sql:
+            filters = (storage_type,)
             request = "SELECT rowid FROM tracks\
                        WHERE storage_type & ?"
             if not skipped:
-                request += " AND loved != -1 "
+                request += " AND not loved &? "
+                filters += (LovedFlags.SKIPPED,)
             request += " ORDER BY album_id"
-            result = sql.execute(request, (storage_type,))
+            result = sql.execute(request, filters)
             return list(itertools.chain(*result))
 
     def get_ids_for_name(self, name):
@@ -215,25 +217,6 @@ class TracksDatabase:
                 return v[0]
             return None
 
-    def get_year_for_album(self, album_id):
-        """
-            Get album year based on tracks
-            Use most used year by tracks
-            @param album_id as int
-            @return int
-        """
-        with SqlCursor(self.__db) as sql:
-            result = sql.execute("SELECT year, COUNT(year) AS occurrence\
-                                  FROM tracks\
-                                  WHERE tracks.album_id=?\
-                                  GROUP BY year\
-                                  ORDER BY occurrence DESC\
-                                  LIMIT 1", (album_id,))
-            v = result.fetchone()
-            if v is not None:
-                return v[0]
-            return None
-
     def get_timestamp_for_album(self, album_id):
         """
             Get album timestamp based on tracks
@@ -662,7 +645,8 @@ class TracksDatabase:
                                            "OR",
                                            len(artist_ids))
             if not skipped:
-                request += " AND loved != -1 "
+                request += " AND not loved &? "
+                filters += (LovedFlags.SKIPPED,)
             filters += (limit,)
             request += " ORDER BY popularity DESC LIMIT ?"
             result = sql.execute(request, filters)
@@ -683,7 +667,8 @@ class TracksDatabase:
                                                "OR",
                                                len(artist_ids))
                 if not skipped:
-                    request += " AND loved != -1 "
+                    request += " AND not loved &? "
+                    filters += (LovedFlags.SKIPPED,)
                 filters += (limit,)
                 request += " ORDER BY popularity DESC LIMIT ?"
                 result = sql.execute(request, filters)
@@ -772,12 +757,27 @@ class TracksDatabase:
             @return tracks as [int]
         """
         with SqlCursor(self.__db) as sql:
+            filters = (storage_type,)
             request = "SELECT tracks.rowid FROM tracks\
                        WHERE ltime!=0 AND storage_type & ?"
             if not skipped:
-                request += " AND loved != -1 "
+                request += " AND not loved &? "
+                filters += (LovedFlags.SKIPPED,)
             request += " ORDER BY ltime DESC LIMIT ?"
-            result = sql.execute(request, (storage_type, limit))
+            filters += (limit,)
+            result = sql.execute(request, filters)
+            return list(itertools.chain(*result))
+
+    def get_skipped(self, storage_type):
+        """
+            Return skipped tracks
+            @param storage_type as StorageType
+            @return tracks as [int]
+        """
+        with SqlCursor(self.__db) as sql:
+            request = "SELECT rowid FROM tracks\
+                       WHERE loved=-1 AND storage_type & ?"
+            result = sql.execute(request, (storage_type,))
             return list(itertools.chain(*result))
 
     def get_randoms(self, genre_ids, storage_type, skipped, limit):
@@ -796,7 +796,8 @@ class TracksDatabase:
                 request += ",track_genres"
             request += " WHERE storage_type & ? "
             if not skipped:
-                request += " AND loved != -1 "
+                request += " AND not loved &? "
+                filters += (LovedFlags.SKIPPED,)
             if genre_ids:
                 request += "AND tracks.rowid=track_genres.track_id"
                 filters += tuple(genre_ids)
@@ -848,6 +849,30 @@ class TracksDatabase:
                 return v[0]
             return 0
 
+    def get_year_for_album(self, album_id, disc_number=None):
+        """
+            Get album year based on tracks
+            Use most used year by tracks
+            @param album_id as int
+            @param disc_number as int/None
+            @return int
+        """
+        with SqlCursor(self.__db) as sql:
+            filters = (album_id,)
+            request = "SELECT year, COUNT(year) AS occurrence FROM tracks\
+                       WHERE tracks.album_id=?"
+            if disc_number is not None:
+                filters += (disc_number,)
+                request += " AND tracks.discnumber=?"
+            request += " GROUP BY year\
+                        ORDER BY occurrence DESC"
+            result = sql.execute(request, filters)
+            v = list(result)
+            # Ignore album with multiple original date
+            if len(v) == 1:
+                return v[0][0]
+            return None
+
     def get_ltime(self, track_id):
         """
             Get listen time
@@ -877,6 +902,92 @@ class TracksDatabase:
                 return v[0]
             return 0
 
+    def get_years(self, storage_type):
+        """
+            Return all tracks years and if unknown years exist
+            @param storage_type as StorageType
+            @return ([int], bool)
+        """
+        with SqlCursor(self.__db) as sql:
+            result = sql.execute("SELECT tracks.year\
+                                  FROM tracks\
+                                  WHERE storage_type & ?",
+                                 (storage_type,))
+            years = []
+            unknown = False
+            for year in list(itertools.chain(*result)):
+                if year is None:
+                    unknown = True
+                elif year not in years:
+                    years.append(year)
+            return (years, unknown)
+
+    def get_albums_by_disc_for_year(self, year, storage_type,
+                                    skipped, limit=-1):
+        """
+            Return albums for year
+            @param year as int
+            @param storage_type as StorageType
+            @param skipped as bool
+            @param limit as int
+            @return discs [(int, int)]
+        """
+        with SqlCursor(self.__db) as sql:
+            order = " ORDER BY artists.sortname\
+                     COLLATE NOCASE COLLATE LOCALIZED,\
+                     tracks.timestamp,\
+                     albums.name\
+                     COLLATE NOCASE COLLATE LOCALIZED LIMIT ?"
+            request = "SELECT DISTINCT tracks.album_id,\
+                       discnumber,\
+                       discname,\
+                       albums.year\
+                       FROM albums, tracks, album_artists, artists\
+                       WHERE albums.rowid=album_artists.album_id AND\
+                       artists.rowid=album_artists.artist_id AND\
+                       tracks.album_id=albums.rowid AND\
+                       tracks.year=? AND albums.storage_type & ?"
+            filters = (year, storage_type, limit)
+            if not skipped:
+                request += " AND not albums.loved &? "
+                filters += (LovedFlags.SKIPPED,)
+            request += " GROUP BY tracks.album_id"
+            request += order
+            result = sql.execute(request, filters)
+            return list(result)
+
+    def get_compilations_by_disc_for_year(self, year, storage_type,
+                                          skipped, limit=-1):
+        """
+            Return compilations for year
+            @param year as int
+            @param storage_type as StorageType
+            @param skipped as bool
+            @param limit as int
+            @return discs [(int, int)]
+        """
+        with SqlCursor(self.__db) as sql:
+            order = " ORDER BY albums.timestamp, albums.name\
+                     COLLATE NOCASE COLLATE LOCALIZED LIMIT ?"
+            request = "SELECT DISTINCT tracks.album_id,\
+                       discnumber,\
+                       discname,\
+                       albums.year\
+                       FROM albums, album_artists, tracks\
+                       WHERE album_artists.artist_id=?\
+                       AND album_artists.album_id=albums.rowid\
+                       AND tracks.album_id=albums.rowid\
+                       AND albums.storage_type & ?\
+                       AND tracks.year=?"
+            filters = (Type.COMPILATIONS, storage_type, year, limit)
+            if not skipped:
+                request += " AND not albums.loved &? "
+                filters += (LovedFlags.SKIPPED,)
+            request += " GROUP BY tracks.album_id"
+            request += order
+            result = sql.execute(request, filters)
+            return list(result)
+
     def set_lp_track_id(self, track_id, lp_track_id):
         """
             Set lp track id
