@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -10,9 +10,9 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, GLib
+from gi.repository import Gtk, GLib, Handy
 
-from lollypop.define import App, Size, AdaptiveSize
+from lollypop.define import App, Size
 from lollypop.toolbar_playback import ToolbarPlayback
 from lollypop.objects_track import Track
 from lollypop.toolbar_info import ToolbarInfo
@@ -22,7 +22,7 @@ from lollypop.helper_size_allocation import SizeAllocationHelper
 from lollypop.helper_signals import SignalsHelper, signals
 
 
-class Toolbar(Gtk.HeaderBar, SizeAllocationHelper, SignalsHelper):
+class Toolbar(Handy.HeaderBar, SizeAllocationHelper, SignalsHelper):
     """
         Lollypop toolbar
     """
@@ -37,7 +37,6 @@ class Toolbar(Gtk.HeaderBar, SizeAllocationHelper, SignalsHelper):
         SizeAllocationHelper.__init__(self)
         self.__width = Size.MINI
         self.__timeout_id = None
-        self.__adaptive_size = AdaptiveSize.PHONE
         self.set_title("Lollypop")
         self.__toolbar_playback = ToolbarPlayback(window)
         self.__toolbar_playback.show()
@@ -52,8 +51,8 @@ class Toolbar(Gtk.HeaderBar, SizeAllocationHelper, SignalsHelper):
         self.pack_end(self.__toolbar_end)
         return [
             (App().player, "current-changed", "_on_current_changed"),
-            (App().window, "adaptive-size-changed",
-             "_on_adaptive_size_changed")
+            (App().window.container.widget, "notify::folded",
+             "_on_container_folded")
         ]
 
     def do_get_preferred_width(self):
@@ -64,6 +63,31 @@ class Toolbar(Gtk.HeaderBar, SizeAllocationHelper, SignalsHelper):
         width = max(Size.PHONE, self.__width)
         return (Size.PHONE, width)
 
+    def hide_info_and_buttons(self, folded):
+        """
+            Hide information and buttons
+            @param folded as bool
+        """
+        def show_children():
+            self.__timeout_id = None
+            self.__toolbar_info.show_children()
+
+        if self.__timeout_id is not None:
+            GLib.source_remove(self.__timeout_id)
+            self.__timeout_id = None
+
+        if folded:
+            self.__toolbar_playback.player_buttons.hide()
+            self.__toolbar_title.hide()
+            self.__toolbar_info.hide_children()
+        else:
+            if App().player.current_track.id is not None:
+                self.__toolbar_title.show()
+            # If user double click headerbar to maximize window
+            # We do not want info bar to receive click signal
+            self.__timeout_id = GLib.timeout_add(200, show_children)
+            self.__toolbar_playback.player_buttons.show()
+
     @property
     def end(self):
         """
@@ -121,7 +145,7 @@ class Toolbar(Gtk.HeaderBar, SizeAllocationHelper, SignalsHelper):
             @param player as Player
         """
         if player.current_track.id is not None and\
-                self.__adaptive_size & (AdaptiveSize.BIG | AdaptiveSize.LARGE):
+                not App().window.folded:
             if isinstance(player.current_track, Track):
                 self.__toolbar_title.show()
             else:
@@ -131,30 +155,10 @@ class Toolbar(Gtk.HeaderBar, SizeAllocationHelper, SignalsHelper):
             self.__toolbar_title.hide()
             self.__toolbar_info.hide_children()
 
-    def _on_adaptive_size_changed(self, window, adaptive_size):
+    def _on_container_folded(self, leaflet, folded):
         """
             Update internal widgets
-            @param window as Gtk.Window
-            @param adaptive_size as AdaptiveSize
+            @param leaflet as Handy.Leaflet
+            @param folded as Gparam
         """
-        def show_children():
-            self.__timeout_id = None
-            self.__toolbar_info.show_children()
-
-        if self.__timeout_id is not None:
-            GLib.source_remove(self.__timeout_id)
-            self.__timeout_id = None
-
-        self.__adaptive_size = adaptive_size
-        if adaptive_size & (AdaptiveSize.BIG | AdaptiveSize.LARGE):
-            if App().player.current_track.id is not None:
-                if isinstance(App().player.current_track, Track):
-                    self.__toolbar_title.show()
-                # If user double click headerbar to maximize window
-                # We do not want info bar to receive click signal
-                self.__timeout_id = GLib.timeout_add(200, show_children)
-            self.__toolbar_playback.player_buttons.show()
-        else:
-            self.__toolbar_playback.player_buttons.hide()
-            self.__toolbar_title.hide()
-            self.__toolbar_info.hide_children()
+        self.hide_info_and_buttons(App().window.folded)
