@@ -1,4 +1,4 @@
-# Copyright (c) 2014-2020 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
+# Copyright (c) 2014-2021 Cedric Bellegarde <cedric.bellegarde@adishatz.org>
 # This program is free software: you can redistribute it and/or modify
 # it under the terms of the GNU General Public License as published by
 # the Free Software Foundation, either version 3 of the License, or
@@ -10,18 +10,20 @@
 # You should have received a copy of the GNU General Public License
 # along with this program. If not, see <http://www.gnu.org/licenses/>.
 
-from gi.repository import Gtk, Pango
+from gi.repository import Gtk, Pango, GLib
+
+from gettext import gettext as _
 
 from lollypop.define import App, ArtSize, Type, ViewType
 from lollypop.define import ArtBehaviour, MARGIN_MEDIUM, MARGIN, MARGIN_SMALL
 from lollypop.widgets_rating import RatingWidget
 from lollypop.widgets_loved import LovedWidget
+from lollypop.widgets_label import LabelWidget
 from lollypop.widgets_cover import EditCoverWidget, CoverWidget
 from lollypop.widgets_banner import BannerWidget
 from lollypop.utils import get_human_duration, on_query_tooltip
-from lollypop.utils import set_cursor_type, popup_widget, emit_signal
+from lollypop.utils import popup_widget, emit_signal
 from lollypop.helper_signals import SignalsHelper, signals_map
-from lollypop.helper_gestures import GesturesHelper
 
 
 class AlbumBannerWidget(BannerWidget, SignalsHelper):
@@ -44,7 +46,7 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
         return [
                 (App().window.container.widget, "notify::folded",
                  "_on_container_folded"),
-                (App().art, "album-artwork-changed",
+                (App().album_art, "album-artwork-changed",
                  "_on_album_artwork_changed"),
                 (App().player, "playback-added", "_on_playback_changed"),
                 (App().player, "playback-updated", "_on_playback_changed"),
@@ -60,7 +62,7 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
             return
         self.__widget = Gtk.Grid.new()
         self.__widget.set_margin_start(MARGIN)
-        self.__widget.set_margin_end(MARGIN_MEDIUM)
+        self.__widget.set_margin_end(MARGIN)
         self.__widget.set_margin_top(MARGIN_SMALL)
         self.__widget.set_margin_bottom(MARGIN_SMALL)
         self.__widget.set_row_spacing(MARGIN_SMALL)
@@ -74,15 +76,12 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
         self.__middle_box.set_valign(Gtk.Align.CENTER)
         self.__middle_box.set_hexpand(True)
         self.__bottom_box.set_vexpand(True)
-        self.__year_eventbox = Gtk.EventBox.new()
-        self.__year_eventbox.set_hexpand(True)
-        self.__year_eventbox.set_halign(Gtk.Align.END)
-        self.__year_label = Gtk.Label.new()
+        self.__year_label = LabelWidget()
+        self.__year_label.connect("clicked", self.__on_year_clicked)
+        self.__year_label.set_justify(Gtk.Justification.RIGHT)
         self.__year_label.get_style_context().add_class("dim-label")
-        self.__year_eventbox.add(self.__year_label)
-        self.__year_eventbox.connect("realize", set_cursor_type)
-        self.__gesture_year = GesturesHelper(
-            self.__year_eventbox, primary_press_callback=self._on_year_press)
+        self.__year_label.set_halign(Gtk.Align.END)
+        self.__year_label.set_hexpand(True)
         self.__play_button = Gtk.Button.new_from_icon_name(
             "media-playback-start-symbolic", Gtk.IconSize.BUTTON)
         self.__add_button = Gtk.Button.new_from_icon_name(
@@ -99,29 +98,26 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
         self.__title_label.set_ellipsize(Pango.EllipsizeMode.END)
         self.__title_label.set_halign(Gtk.Align.START)
         self.__title_label.set_xalign(0)
-        self.__artist_label = Gtk.Label.new()
-        self.__artist_eventbox = Gtk.EventBox.new()
-        self.__artist_eventbox.set_valign(Gtk.Align.START)
-        self.__artist_eventbox.add(self.__artist_label)
-        self.__artist_eventbox.connect("realize", set_cursor_type)
+        self.__artist_label = LabelWidget()
+        self.__artist_label.show()
+        self.__artist_label.connect("clicked", self.__on_artist_clicked)
+        self.__artist_label.set_valign(Gtk.Align.START)
+        self.__artist_label.set_halign(Gtk.Align.START)
         self.__artist_label.connect("query-tooltip", on_query_tooltip)
         self.__artist_label.set_ellipsize(Pango.EllipsizeMode.END)
-        self.__artist_label.set_halign(Gtk.Align.START)
         self.__artist_label.set_xalign(0)
-        self.__gesture_artist = GesturesHelper(
-            self.__artist_eventbox,
-            primary_press_callback=self._on_artist_press)
         self.__duration_label = Gtk.Label.new()
         self.__duration_label.get_style_context().add_class("dim-label")
-        self.__top_box.pack_end(self.__year_eventbox, False, True, 0)
+        self.__top_box.pack_end(self.__year_label, False, True, 0)
         self.__middle_box.add(self.__play_button)
         self.__middle_box.add(self.__add_button)
         self.__middle_box.add(self.__menu_button)
         self.__middle_box.get_style_context().add_class("linked")
         self.__bottom_box.pack_end(self.__duration_label, False, True, 0)
         self.__labels_box.add(self.__title_label)
-        self.__labels_box.add(self.__artist_eventbox)
+        self.__labels_box.add(self.__artist_label)
         self.__widget.attach(self.__top_box, 2, 0, 1, 1)
+        self.__set_album_year(True)
         self.__widget.attach(self.__middle_box, 2, 1, 1, 1)
         self.__widget.attach(self.__bottom_box, 1, 2, 2, 1)
         self.__widget.attach(self.__labels_box, 1, 0, 1, 2)
@@ -146,9 +142,6 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
             self.__cover_widget.connect("populated",
                                         self.__on_cover_populated)
         self.__cover_widget.show()
-        if self.__album.year is not None:
-            self.__year_label.set_label(str(self.__album.year))
-            self.__year_label.show()
         human_duration = get_human_duration(self.__album.duration)
         self.__duration_label.set_text(human_duration)
         self.__widget.attach(self.__cover_widget, 0, 0, 1, 3)
@@ -179,10 +172,19 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
             Mark widget as selected
             @param selected as bool
         """
-        if selected:
-            self.__widget.set_state_flags(Gtk.StateFlags.SELECTED, True)
-        else:
-            self.__widget.set_state_flags(Gtk.StateFlags.NORMAL, True)
+        buttons = [self.__play_button, self.__add_button, self.__menu_button]
+        for button in buttons:
+            if selected:
+                button.set_state_flags(Gtk.StateFlags.SELECTED, True)
+            else:
+                button.set_state_flags(Gtk.StateFlags.NORMAL, True)
+
+    def update_duration(self):
+        """
+            Update duration label
+        """
+        human_duration = get_human_duration(self.__album.duration)
+        self.__duration_label.set_text(human_duration)
 
 #######################
 # PROTECTED           #
@@ -218,25 +220,6 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
         """
         self.__update_add_button()
 
-    def _on_year_press(self, x, y, event):
-        """
-            Show year view
-            @param x as int
-            @param y as int
-            @param event as Gdk.EventButton
-        """
-        App().window.container.show_view([Type.YEARS], [self.__album.year])
-
-    def _on_artist_press(self, x, y, event):
-        """
-            Show artist view
-            @param x as int
-            @param y as int
-            @param event as Gdk.EventButton
-        """
-        App().window.container.show_view([Type.ARTISTS],
-                                         self.__album.artist_ids)
-
 #######################
 # PRIVATE             #
 #######################
@@ -246,6 +229,28 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
         else:
             self.__artwork_popup.destroy()
 
+    def __set_album_year(self, full):
+        """
+            Set album year
+            @param full as int
+        """
+        if self.__album.year is not None:
+            album_year = GLib.markup_escape_text(
+                "%s" % self.__album.year)
+            markup = None
+            if full and self.__album.original_year is not None:
+                original_year = GLib.markup_escape_text(
+                    "%s" % self.__album.original_year)
+                if original_year != album_year:
+                    original_str = _("Released on %s") % original_year
+                    markup = \
+                        "%s\n<span size='x-small' alpha='40000'>%s</span>" %\
+                        (album_year, original_str)
+            if markup is None:
+                markup = album_year
+            self.__year_label.set_markup(markup)
+            self.__year_label.show()
+
     def __set_artwork(self):
         """
             Set artwork on banner
@@ -268,22 +273,31 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
                 self.__widget.remove(self.__top_box)
                 self.__widget.attach(self.__top_box, 1, 1, 1, 1)
                 self.__widget.attach(self.__labels_box, 1, 0, 2, 1)
+                self.__set_album_year(False)
         elif self.__widget.get_child_at(1, 0) == self.__labels_box:
             self.__widget.remove(self.__labels_box)
             self.__widget.remove(self.__top_box)
             self.__widget.attach(self.__top_box, 2, 0, 1, 1)
             self.__widget.attach(self.__labels_box, 1, 0, 1, 2)
+            self.__set_album_year(True)
 
     def __update_add_button(self):
         """
             Set image as +/-
         """
-        if self.__album.id in App().player.album_ids:
-            self.__add_button.get_image().set_from_icon_name(
-                "list-remove-symbolic", Gtk.IconSize.BUTTON)
-        else:
+        albums = App().player.get_albums_for_id(self.__album.id)
+        missing = False if albums else True
+        for album in albums:
+            for track in self.__album.tracks:
+                if track.id not in album.track_ids:
+                    missing = True
+                    break
+        if missing:
             self.__add_button.get_image().set_from_icon_name(
                 "list-add-symbolic", Gtk.IconSize.BUTTON)
+        else:
+            self.__add_button.get_image().set_from_icon_name(
+                "list-remove-symbolic", Gtk.IconSize.BUTTON)
 
     def __set_internal_size(self):
         """
@@ -362,7 +376,36 @@ class AlbumBannerWidget(BannerWidget, SignalsHelper):
             Add/Remove album
            @param button as Gtk.Button
         """
-        if self.__album.id in App().player.album_ids:
-            App().player.remove_album_by_id(self.__album.id)
+        add = self.__add_button.get_image().get_icon_name()[0] ==\
+            "list-add-symbolic"
+        albums = App().player.get_albums_for_id(self.__album.id)
+        if albums:
+            if add:
+                for track in self.__album.tracks:
+                    albums[0].append_track(track)
+                emit_signal(App().player, "playback-updated", albums[0])
+            else:
+                for album in albums:
+                    for track in self.__album.tracks:
+                        album.remove_track(track)
+                        if album.id is None:
+                            App().player.remove_album(album)
+                            break
+                emit_signal(App().player, "playback-updated", album)
         else:
             App().player.add_album(self.__album.clone(False))
+
+    def __on_year_clicked(self, label):
+        """
+            Show year view
+            @param label as LabelWidget
+        """
+        App().window.container.show_view([Type.YEARS], [self.__album.year])
+
+    def __on_artist_clicked(self, label):
+        """
+            Show artist view
+            @param label as LabelWidget
+        """
+        App().window.container.show_view([Type.ARTISTS],
+                                         self.__album.artist_ids)
